<?php
namespace Wpexperts\AddressAutocompleteForWoocommerce\Admin;

class GoogleMapApiSetting {

	public function __construct() {
		// register Map options
		add_action( 'admin_init', array( $this, 'map_option_options' ) );

		// add Map settings
		add_action( 'admin_menu', array( $this, 'map_option_settings' ) );
	}

	/**
	 * Add Map settings
	 */
	public function map_option_settings() {
		// add a new menu item. This is a top level menu item i.e., this menu item can have sub menus
		add_submenu_page( 'woocommerce', __( 'Address Autocomplete', 'address-autocomplete-for-woocommerce' ), __( 'Address Autocomplete', 'address-autocomplete-for-woocommerce' ), 'manage_options', 'map-option', array( $this, 'map_option_options_page' ) );
	}

	public function map_option_options_page() {
		?>
			<div class="wrap">
				<h1><?php echo esc_html__( 'Google Map API', 'address-autocomplete-for-woocommerce' ); ?></h1>
				<form method="post" action="options.php">
			<?php
				settings_fields( 'map_fields' );
				do_settings_sections( 'map-options' );
				submit_button();
			?>
				</form>
			</div>
			<?php
	}

	/**
	 * Register Map options
	 */
	public function map_option_options() {
		// section name, display name, callback to print description of section, page to which section is attached.
		add_settings_section( 'map_fields', __( 'Google API Data', 'address-autocomplete-for-woocommerce' ), null, 'map-options' );

		add_settings_field( 'google_map_api_key', __( 'Google API Key', 'address-autocomplete-for-woocommerce' ), array( $this, 'display_google_map_api_key' ), 'map-options', 'map_fields' );
		// section name, form element name, callback for sanitization

		add_settings_field( 'aafw_allow_manual_address', __( 'Disable Plugin functionality', 'address-autocomplete-for-woocommerce' ), array( $this, 'display_manual_address_field' ), 'map-options', 'map_fields' );

		// add setting to create option to disable map from frontend.
		add_settings_field( 'aafw_enable_map', __( 'Enable Map', 'address-autocomplete-for-woocommerce' ), array( $this, 'display_enable_map_field' ), 'map-options', 'map_fields' );

		// add setting to create option to disable map from frontend.
		add_settings_field( 'aafw_enable_restriction', __( 'Selling location', 'address-autocomplete-for-woocommerce' ), array( $this, 'selling_location' ), 'map-options', 'map_fields' );

		add_settings_field(
			'aafw_enable_fieldval',
			__( 'Street address field display', 'address-autocomplete-for-woocommerce' ) . '<span class="woocommerce-help-tip" tabindex="0" aria-label="' . esc_attr__( 'If all fields are unchecked, all fields will be displayed by default.', 'address-autocomplete-for-woocommerce' ) . '"></span>',
			array( $this, 'selling_fdisplay' ),
			'map-options',
			'map_fields'
		);
		add_settings_field(
			'aafw_map_zoom',
			__( 'Custom Map Zoom', 'address-autocomplete-for-woocommerce' ) . '<span class="woocommerce-help-tip" tabindex="0" aria-label="' . esc_attr__( "If you don't select a map zoom value, the default setting of 8 will be applied.", 'address-autocomplete-for-woocommerce' ) . '"></span>',
			array( $this, 'zoom_map_display' ),
			'map-options',
			'map_fields'
		);
		add_settings_field('map_style', __('Map Style', 'address-autocomplete-for-woocommerce'), array( $this, 'display_map_style_field' ), 'map-options', 'map_fields');

		register_setting('map_fields', 'google_map_api_key');
		register_setting('map_fields', 'aafw_allow_manual_address');
		register_setting('map_fields', 'aafw_enable_map');
		register_setting('map_fields', 'Postalcc');
		register_setting('map_fields', 'administrative_area_level_1addr');
		register_setting('map_fields', 'administrative_area_level_2addr');
		register_setting('map_fields', 'street_numberrouteaddr');
		register_setting('map_fields', 'localityaddr');
		register_setting('map_fields', 'neighborhoodaddr');
		register_setting('map_fields', 'routeaddr');
		register_setting('map_fields', 'Countryaddr');
		register_setting('map_fields', 'aafw_enable_restriction');
		register_setting('map_fields', 'aafw_enable_fieldval');
		register_setting('map_fields', 'map_style');
		register_setting('map_fields', 'custom_restriction_msg');
		register_setting('map_fields', 'custom_zoom_map');
	}
	public function zoom_map_display() {
		$custom_zoom_map = get_option('custom_zoom_map', 8); // Retrieve custom map zoom level
		?>
			<!-- <p>
				<label for="custom_zoom_map"><?php /*esc_html_e( 'Custom Map Zoom:', 'address-autocomplete-for-woocommerce' );*/ ?></label>
			</p> -->
			<p>
				<select id="custom_zoom_map" name="custom_zoom_map" style="width: 100px;">
				<?php for ($i = 0; $i <= 18; $i++) : ?>
						<option value="<?php echo esc_attr($i); ?>" <?php selected($custom_zoom_map, $i); ?>>
							<?php echo esc_html($i); ?>
						</option>
					<?php endfor; ?>
				</select>
			</p>
			<?php
	}
		
	public function display_map_style_field() {
		$map_style = get_option('map_style', 1); // Default map style

		// Map styles and their corresponding labels and images
		$map_styles = array(
			1 => array( 'label' => __('Standard Map', 'address-autocomplete-for-woocommerce'), 'image' => '1.png' ),
			2 => array( 'label' => __('Silver Map', 'address-autocomplete-for-woocommerce'), 'image' => '2.png' ),
			3 => array( 'label' => __('Retro Map', 'address-autocomplete-for-woocommerce'), 'image' => '3.png' ),
			4 => array( 'label' => __('Dark Map', 'address-autocomplete-for-woocommerce'), 'image' => '4.png' ),
			5 => array( 'label' => __('Night Map', 'address-autocomplete-for-woocommerce'), 'image' => '5.png' ),
			6 => array( 'label' => __('Aubergine Map', 'address-autocomplete-for-woocommerce'), 'image' => '6.png' ),
			7 => array( 'label' => __('Basic Map', 'address-autocomplete-for-woocommerce'), 'image' => '7.png' ),
		);

		register_setting('map_fields', 'map_style'); // Ensure this is set up correctly in your settings

		// Display the dropdown, labels, and corresponding images
		echo '<div class="map-styles-container">';
		// echo '<label for="map-style-select">' . esc_html__('Select Map Style', 'address-autocomplete-for-woocommerce') . '</label>';
		echo '<br>';
		echo '<select id="map-style-select" name="map_style">';
		foreach ($map_styles as $key => $style) {
			echo '<option value="' . esc_html__($key) . '" ' . selected($map_style, $key, false) . '>' . esc_html($style['label']) . '</option>';
		}
		echo '</select>';
		echo '</div>';

		// Display the map image below the dropdown
		echo '<div id="map-image-container">';
		echo '<img id="map-image" src="' . esc_url(WC_ADDRESS_AUTOCOMPLETE_URL) . 'assets/img/' . esc_attr($map_styles[$map_style]['image']) . '" />';
		echo '</div>';

		// Add CSS styles to position the map image and make it larger
		echo '<style>';
		echo '.map-styles-container {  }';
		echo '#map-image-container { margin-top: 20px; }';
		echo '#map-image { width: 50%; max-width: 200px; height: auto; }'; // Adjust the max-width and other styles as needed
		echo '</style>';
	}

		
	public function display_google_map_api_key() {
		$google_map_api_key = get_option( 'google_map_api_key' );
		?>
			<input type="text" name="google_map_api_key" value="<?php echo esc_attr_e( $google_map_api_key, 'address-autocomplete-for-woocommerce' ); ?>">
			<?php
	}

	public function display_manual_address_field() {
		$aafw_allow_manual_address = get_option( 'aafw_allow_manual_address' );
		?>
			<input type="checkbox" id="aafw_allow_manual_address" name="aafw_allow_manual_address" value="1" <?php checked( 1, $aafw_allow_manual_address, true ); ?> />
			<span><?php esc_html_e( 'Enable this to disable the functionality of Autocomplete.', 'address-autocomplete-for-woocommerce' ); ?></span>
			<?php
	}

	public function display_enable_map_field() {
		$aafw_enable_map = get_option( 'aafw_enable_map', 1 );
		?>
			<input type="checkbox" id="aafw_enable_map" name="aafw_enable_map" value="1" <?php checked( 1, $aafw_enable_map, true ); ?> />
			<span><?php esc_html_e( 'Enable this, if you want to show map on checkout page. Addresses will be validated by google maps.', 'address-autocomplete-for-woocommerce' ); ?></span>
			<?php
	}
	public function selling_fdisplay() {
			
		$Postalcc                        = get_option( 'Postalcc', 1 );
		$Countryaddr                     = get_option( 'Countryaddr', 1 );
		$administrative_area_level_1addr = get_option( 'administrative_area_level_1addr', 1 );
		$administrative_area_level_2addr = get_option( 'administrative_area_level_2addr', 1 );
		$localityaddr                    = get_option( 'localityaddr', 1 );
		$neighborhoodaddr                = get_option( 'neighborhoodaddr', 1 );
		$routeaddr                       = get_option( 'routeaddr', 1 );
		$street_numberrouteaddr          = get_option( 'street_numberrouteaddr', 1 );
		?>
			
			<label><input type="checkbox" id="street_numberrouteaddr" class="street_numberrouteaddr" name="street_numberrouteaddr" value="1" <?php checked(1, $street_numberrouteaddr, true); ?> /><?php esc_html_e('Street Number', 'address-autocomplete-for-woocommerce'); ?></label>
			</br>
			
			<label><input type="checkbox" id="routeaddr" class="routeaddr" name="routeaddr" value="1" <?php checked(1, $routeaddr, true); ?> /><?php esc_html_e('Route', 'address-autocomplete-for-woocommerce'); ?></label>
			</br>
			
			<label><input type="checkbox" id="neighborhoodaddr" class="neighborhoodaddr" name="neighborhoodaddr" value="1" <?php checked(1, $neighborhoodaddr, true); ?> /><?php esc_html_e('Neighbourhood', 'address-autocomplete-for-woocommerce'); ?></label>
			</br>
			
			<label><input type="checkbox" id="localityaddr" class="localityaddr" name="localityaddr" value="1" <?php checked(1, $localityaddr, true); ?> /><?php esc_html_e('Locality', 'address-autocomplete-for-woocommerce'); ?></label>
			</br>
			
			<label><input type="checkbox" id="administrative_area_level_2addr" class="administrative_area_level_2addr" name="administrative_area_level_2addr" value="1" <?php checked(1, $administrative_area_level_2addr, true); ?> /><?php esc_html_e('Administrative Area Level 2', 'address-autocomplete-for-woocommerce'); ?></label>
			</br>
			
			<label><input type="checkbox" id="administrative_area_level_1addr" class="administrative_area_level_1addr" name="administrative_area_level_1addr" value="1" <?php checked(1, $administrative_area_level_1addr, true); ?> /><?php esc_html_e('Administrative Area Level 1', 'address-autocomplete-for-woocommerce'); ?></label>
			</br>
			
			<label><input type="checkbox" id="Countryaddr" class="Countryaddr" name="Countryaddr" value="1" <?php checked(1, $Countryaddr, true); ?> /><?php esc_html_e('Country, Political', 'address-autocomplete-for-woocommerce'); ?></label>
			</br>
			
			<label><input type="checkbox" id="Postalcc" class="Postalcc" name="Postalcc" value="1" <?php checked(1, $Postalcc, true); ?> /><?php esc_html_e('Postal Code', 'address-autocomplete-for-woocommerce'); ?></label>
			</br>
			<?php
	}
	public function selling_location() {
		$aafw_enable_restriction = get_option('aafw_enable_restriction', 1);
		$custom_restriction_msg  = get_option('custom_restriction_msg'); // Retrieve custom message
		?>
			<input type="checkbox" id="aafw_enable_restriction" class="aafw_enable_restriction" name="aafw_enable_restriction" value="1" <?php checked(1, $aafw_enable_restriction, true); ?> />
			<span><?php esc_html_e('Enable this, to restrict the result of address autocomplete to specific countries. If no countries are selected result will not be restricted.', 'address-autocomplete-for-woocommerce'); ?></span>

			<p>
				<label for="custom_restriction_msg"><?php esc_html_e( 'Custom Restriction Message:', 'address-autocomplete-for-woocommerce' ); ?></label>
			</p>
			<p>
				<input type="text" id="custom_restriction_msg" name="custom_restriction_msg" value="<?php echo esc_attr_e($custom_restriction_msg, 'address-autocomplete-for-woocommerce' ); ?>" style="width: 500px;" />
			</p>
			<p>
				<strong><?php esc_html_e('Note:', 'address-autocomplete-for-woocommerce'); ?></strong>
				<?php esc_html_e('You can only select up to 5 countries in WooCommerce->General settings for specific restrictions due to Google API filters restriction.', 'address-autocomplete-for-woocommerce'); ?>
			</p>
			<?php
	}
}
	