<?php
namespace Wpexperts\AddressAutocompleteForWoocommerce\Frontend;

/**
 * Class use woocommerce hooks to add custom code related to map_checkout shipping
 */
class WcFrontendHooks {


	public function __construct() {
		// enqueue plugin style and scripts
		add_action('wp_enqueue_scripts', array( $this, 'register_map_checkout_styles' ));
		// // enqueue admin scripts
		add_action('admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ));
		// //       add lat and lang admin hidden fields
		add_action('woocommerce_admin_order_data_after_order_details', array( $this, 'address_hidden_fields' ));
		// //save admin billing address
		add_action('woocommerce_process_shop_order_meta', array( $this, 'admin_field_update_order_meta' ), 50, 1);

		// // save address from map in edit address
		add_action('woocommerce_customer_save_address', array( $this, 'my_account_saving_billing_address' ), 10, 1);
		// // Define map address field structure
		if ('' == get_option('aafw_allow_manual_address')) {
			add_filter('woocommerce_form_field_address_map', array( $this, 'address_map_form_field' ), 50, 3);
		
			add_filter('woocommerce_form_field_shipping_address_map', array( $this, 'shipping_address_map_form_field' ), 50, 3);
		}
		
		// // Update order with new fields meta
		// //add_action('woocommerce_checkout_update_order_meta', array($this, 'customize_checkout_field_update_order_meta'), 999);

		// // Add address map data from submitted form
		add_filter('woocommerce_checkout_posted_data', array( $this, 'wc_checkout_address_map_data' ));
		add_action('woocommerce_checkout_order_processed', array( $this, 'save_address_meta' ), 10, 1);
		// // Validate custom billing and shipping address
		add_action('woocommerce_checkout_process', array( $this, 'wc_validate_billing_shipping_address' ));
		add_filter('woocommerce_checkout_fields', array( $this, 'change_billing_fields' ), 5, 1);
		add_action('woocommerce_checkout_update_order_meta', array( $this, 'street_checkout_field_update_order_meta' ));
		add_filter('woocommerce_localisation_address_formats', array( $this, 'woo_address_appartment_no' ));
		add_filter('woocommerce_formatted_address_replacements', array( $this, 'woo_address_format_funct' ));
		//add nonce field on checkout page.
		add_action('woocommerce_checkout_before_customer_details', array( $this, 'woo_address_nonce_checkout_field' ), 10);
		add_action('woocommerce_before_edit_address_form_billing', array( $this, 'woo_address_nonce_checkout_field' ), 10);
		add_action('woocommerce_before_edit_address_form_shipping', array( $this, 'woo_address_nonce_checkout_field' ), 10);
		add_action('edit_form_top', array( $this, 'woo_address_nonce_checkout_field' ), 10);
		add_action('show_user_profile', array( $this, 'woo_address_nonce_checkout_field' ));
		add_action('edit_user_profile', array( $this, 'woo_address_nonce_checkout_field' ));
	}

	public function woo_address_nonce_checkout_field() {
		wp_nonce_field('_wooaddr_nonce', '_wooaddr_nonce', true, true);
	}

	/**
	 * Enqueue plugin style and scripts
	 */
	public function register_map_checkout_styles() {
		global $wpdb;

		/**
		 * Filters the details of the language assigned to a post.
		 *
		 * Allows modification of the language details for a post.
		 *
		 * @since  1.0.0
		 * @param  array|null $language_details The language details for the post. Default is null.
		 * @return array|null Modified language details or null.
		 */
		$my_post_language_details = apply_filters('wpml_post_language_details', null);
		$google_map_api_key       = get_option('google_map_api_key', null);

		if ( null === $google_map_api_key ) {
			return false;
		}

		if ( '' === empty( trim($google_map_api_key) ) ) {
			return false;
		}
		
		$aafw_disable_plugin = get_option('aafw_allow_manual_address');
		
		if ($aafw_disable_plugin) {
			return false;
		}
		
		// check if is checkout page then enqueue map style and script
		if (( function_exists('is_checkout') && is_checkout() ) || is_page('my-account') ) {
			$post_id      = get_the_ID();
			$selling_location = get_option('woocommerce_allowed_countries');
			$allowed_countries = array();
			$excluded_countries = array();
			if ('specific' === $selling_location ) {
				// Get specific allowed countries
				$allowed_countries = get_option('woocommerce_specific_allowed_countries');
			} elseif ('all_except' === $selling_location ) {
				// Get all countries and exclude the specified ones
				$excluded_countries = get_option('woocommerce_all_except_countries');
				
				// Get all countries (key-value pair of country code and name)
				$all_countries = WC()->countries->get_countries();
				
				// Remove excluded countries
				$allowed_countries = array_diff_key($all_countries, array_flip($excluded_countries));
				
				// Extract only the country codes (keys) from the array
				$allowed_countries = array_keys($allowed_countries);
				
			}
			
			if ('all' !== $selling_location) {
				$custom_msg = get_option('custom_restriction_msg', '');
			} else {
				$custom_msg = '';
			}
			if (has_block('woocommerce/checkout', $post_id)) {
				$mapJS = 'map-script-block.js';
			} else {
				$mapJS = 'map-script.js';
			}

			if (!empty($my_post_language_details) && !empty($my_post_language_details['language_code'])) {
				$current_language = $my_post_language_details['language_code'];
				wp_enqueue_script('google-map', '//maps.googleapis.com/maps/api/js?key=' . trim($google_map_api_key) . '&language=' . $current_language . '&libraries=places,geometry', array(), SHIPPING_WORKSHOP_VERSION, false);
			} else {
				wp_enqueue_script('google-map', '//maps.googleapis.com/maps/api/js?key=' . trim($google_map_api_key) . '&libraries=places,geometry', array(), SHIPPING_WORKSHOP_VERSION, false);
			}
			wp_enqueue_script('map-script', WC_ADDRESS_AUTOCOMPLETE_URL . 'assets/Public/js/' . $mapJS . '?rand=' . wp_rand(), array( 'google-map' ), SHIPPING_WORKSHOP_VERSION, true);
			wp_localize_script(
				'map-script', 'countries', 
				array( 
				'countries' => $allowed_countries,
				'map_display' => get_option('aafw_enable_map', 1), 
				'map_validation' => get_option('aafw_allow_manual_address'), 
				'enable_restriction' => get_option('aafw_enable_restriction'),
				'aafw_enable_map' => get_option('aafw_enable_map'),
				'map_style' => get_option('map_style', 1),
				'custom_msg' => $custom_msg,
				'custom_zoom_map' => get_option('custom_zoom_map', 8),
				'selling_location' => $selling_location,
				)
			);

			// first check 'Shipping location(s)'.
			$shipping_locations = get_option('woocommerce_ship_to_countries');
			$shipping_countries = get_option('woocommerce_specific_allowed_countries'); // ship to countries you sell to (default)
			if ('all' === $shipping_locations || 'disabled' === $shipping_locations || empty(get_option('aafw_enable_restriction')) ) { 
				// ship to all countries
				$shipping_countries = 'all';
			} elseif ('specific' === $shipping_locations ) {
				$shipping_countries = get_option('woocommerce_specific_ship_to_countries');
			}

			// if it is set to
			wp_localize_script('map-script', 'ship_countries', array( 'ship_countries' => $shipping_countries ));
			wp_localize_script('map-script', 'aafw_enable_restriction', array( 'aafw_enable_restriction' => get_option('aafw_enable_restriction') ));
			// get logged in user billing and shipping address data
			$billing_data                = ( is_user_logged_in() && get_user_meta(get_current_user_id(), 'billing_address_map', true) ) ? get_user_meta(get_current_user_id(), 'billing_address_map', true) : array();
			$shipping_data               = ( is_user_logged_in() && get_user_meta(get_current_user_id(), 'shipping_address_map', true) ) ? get_user_meta(get_current_user_id(), 'shipping_address_map', true) : array();
			$back_end_map                = array();
			$woo_default_country         = explode(':', get_option('woocommerce_default_country'));
			$woocommerce_default_country = WC()->countries->countries[ $woo_default_country[0] ];
			
			$fieldsettings = array(
			'Postalcc' => get_option('Postalcc', 1),
			'Countryaddr' => get_option('Countryaddr', 1),
			'administrative_area_level_1addr' => get_option('administrative_area_level_1addr', 1),
			'administrative_area_level_2addr' => get_option('administrative_area_level_2addr', 1),
			'localityaddr' => get_option('localityaddr', 1),
			'neighborhoodaddr' => get_option('neighborhoodaddr', 1),
			'routeaddr' => get_option('routeaddr', 1),
			'street_numberrouteaddr' => get_option('street_numberrouteaddr', 1),
			);
			
			wp_localize_script(
				'map-script',
				'map_data',
				array(
					wp_json_encode(
						array(
							'billing_data'  => $billing_data,
							'shipping_data' => $shipping_data,
							'fieldsettings' => $fieldsettings,
						)
					),
				)
			);
			wp_localize_script('map-script', 'back_end_map', $back_end_map);
			wp_localize_script('map-script', 'woocommerce_default_country', array( 'woocommerce_country' => $woocommerce_default_country ));

			wp_enqueue_style('map-style', WC_ADDRESS_AUTOCOMPLETE_URL . 'assets/Public/css/map-style.css', array(), SHIPPING_WORKSHOP_VERSION, false);   
		}
	}
	public function enqueue_admin_scripts() {

		if (function_exists('get_current_screen') ) {
			$screen = get_current_screen();
			if ('shop_order' === $screen->id ) {
				/**
				 * Filters the details of the language assigned to a post.
				 *
				 * Allows modification of the language details for a post.
				 *
				 * @since 1.0.0
				 *
				 * @param  array|null $language_details The language details for the post. Default is null.
				 * @return array|null Modified language details or null.
				 */
				$my_post_language_details = apply_filters('wpml_post_language_details', null);
				if (!empty($my_post_language_details) && !empty($my_post_language_details['language_code'])) {
					$current_language   = $my_post_language_details['language_code'];
					$google_map_api_key = get_option('google_map_api_key');

					wp_enqueue_script('google-map2', '//maps.googleapis.com/maps/api/js?key=' . $google_map_api_key . '&libraries=places', array(), '1.0', false);
					// enqueue map_checkout admin scripts
					wp_enqueue_script('admin-billing-address-scripts', WC_ADDRESS_AUTOCOMPLETE_URL . 'assets/Public/js/billing_address_script.js', array(), '1.0', false);
				}
			}
			if ('woocommerce_page_map-option' === $screen->id ) {
				wp_enqueue_style('autocomplete_admin_css', WC_ADDRESS_AUTOCOMPLETE_URL . 'assets/Public/css/front.css', array(), '1.0', false);
			}
		}
	}


	// add lat and lang admin hidden fields
	public function address_hidden_fields() {
		?>
		<input hidden="" name="_billing_admin_search_lat" id="_billing_admin_search_lat" />
		<input hidden="" name="_billing_admin_search_lng" id="_billing_admin_search_lng" />
		<?php
	}

	// save address from map in edit address
	public function my_account_saving_billing_address( $user_id ) {

		$billing_address_map_search_input = filter_input(INPUT_POST, 'billing_address_map_search_input', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address = ( !empty($billing_address_map_search_input) ) ? $billing_address_map_search_input : '';

		if (! empty($billing_address) ) {
			$order_address_data = $this->get_order_address_map_data();
			update_user_meta($user_id, 'billing_address_1', $billing_address);
			update_user_meta($user_id, 'billing_address_map', $order_address_data['billing_data']);
		}
	}

	/**
	 * Define map address field structure
	 *
	 * @param  type $field
	 * @param  type $key
	 * @param  type $args
	 * @return string
	 */
	public function address_map_form_field( $field, $key, $args ) {
		$aafw_enable_map           = get_option('aafw_enable_map', 1);
		$aafw_allow_manual_address = get_option('aafw_allow_manual_address', 1);
		$cssClass                  = 'height:auto;margin:0';
		if (1 === $aafw_enable_map ) :
			$cssClass = '';
		endif;
		
		$field  = '<div style="' . $cssClass . '" class="form-row ' . implode(' ', $args['class']) . '" id="' . $args['id'] . '_field">';
		$field .= '<label for="' . $args['id'] . '" class="">' . $args['label'] . '</label>';
		$field .= '<input id="' . $args['id'] . '_search_input" name="' . $args['id'] . '_search_input" class="controls search_input" type="text" placeholder="' . __('Your Address', 'address-autocomplete-for-woocommerce') . '">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_search_lat" name="' . $args['id'] . '_search_lat" class="search_lat">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_search_lng" name="' . $args['id'] . '_search_lng" class="search_lng">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_search_city" name="' . $args['id'] . '_search_city" class="search_city">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_search_country" name="' . $args['id'] . '_search_country" class="search_country">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_find_location" name="' . $args['id'] . '_find_location" class="find_location">';
		$field .= '<div id="' . $args['id'] . '_map" class="address_map"></div>';
		// Add the custom message div
		$custom_msg = get_option('custom_restriction_msg', '');
		$selling_location = get_option('woocommerce_allowed_countries');
			
		if ('all' !== $selling_location) {
			if (! empty($custom_msg) ) {
				$field .= '<div id="custom-message">';
				$field .= '<p style="color: red; font-weight: bold;">' . esc_html__($custom_msg, 'address-autocomplete-for-woocommerce') . '</p>';
				$field .= '</div>';
			}
		}
		$field .= '</div>';

		return $field;
	}

	public function shipping_address_map_form_field( $field, $key, $args ) {
		$aafw_enable_map = get_option('aafw_enable_map', 1);
		$cssClass        = 'height:auto;margin:0';
		if (1 === $aafw_enable_map ) :
			$cssClass = '';
		endif;

		$field  = '<div style="' . $cssClass . '" class="form-row ' . implode(' ', $args['class']) . '" id="' . $args['id'] . '_field">';
		$field .= '<label for="' . $args['id'] . '" class="shipping">' . $args['label'] . '</label>';
		$field .= '<input id="' . $args['id'] . '_search_input" name="' . $args['id'] . '_search_input" class="controls search_input" type="text" placeholder="' . __('Your Address', 'address-autocomplete-for-woocommerce') . '">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_search_lat" name="' . $args['id'] . '_search_lat" class="search_lat">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_search_lng" name="' . $args['id'] . '_search_lng" class="search_lng">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_search_city" name="' . $args['id'] . '_search_city" class="search_city">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_search_country" name="' . $args['id'] . '_search_country" class="search_country">';
		$field .= '<input type="hidden" id="' . $args['id'] . '_find_location" name="' . $args['id'] . '_find_location" class="find_location">';
		$field .= '<div id="' . $args['id'] . '_map" class="shipping_address_map address_map"></div>';
		// Add the custom message div
		$custom_msg = get_option('custom_restriction_msg', '');
		$selling_location = get_option('woocommerce_allowed_countries');
			
		if ('all' !== $selling_location) {
			if (! empty($custom_msg) ) {
				$field .= '<div id="custom-message-shiping">';
				$field .= '<p style="color: red; font-weight: bold;">' . esc_html__($custom_msg, 'address-autocomplete-for-woocommerce') . '</p>';
				$field .= '</div>';
			}
		}

		$field .= '</div>';

		return $field;
	}

	/**
	 * Update order with new fields meta
	 *
	 * @param int $order_id
	 */
	public function customize_checkout_field_update_order_meta( $order_id ) {
		// get billing and shipping address map data
		$order_address_data = $this->get_order_address_map_data();
		wp_die('test');
		// save map billing address
		$billing_data = $order_address_data['billing_data'];
		update_post_meta($order_id, '_billing_address_lat', $billing_data['lat']);
		update_post_meta($order_id, '_billing_address_lng', $billing_data['lng']);
		update_post_meta($order_id, '_billing_map_address', $billing_data['search_input']);
		update_post_meta($order_id, '_billing_map_city', $billing_data['search_city']);
		update_post_meta($order_id, '_billing_map_country', $billing_data['search_country']);
		// save map shipping address
		$data = isset($_SERVER['QUERY_STRING']) ? sanitize_text_field(wp_unslash($_SERVER['QUERY_STRING'])) : '';
		parse_str($data, $query_params);
		if (isset($query_params['ship_to_different_address']) && $query_params['ship_to_different_address']) {
			$shipping_data = $order_address_data['shipping_data'];
			update_post_meta($order_id, '_shipping_address_lat', $shipping_data['lat']);
			update_post_meta($order_id, '_shipping_address_lng', $shipping_data['lng']);
			update_post_meta($order_id, '_shipping_map_address', $shipping_data['search_input']);
			update_post_meta($order_id, '_shipping_map_city', $shipping_data['search_city']);
			update_post_meta($order_id, '_shipping_map_country', $shipping_data['search_country']);
		}
	}

	// save admin billing address
	public function admin_field_update_order_meta( $post_id ) {
		// define variables
		$billing_address = array();
		// get billing data

		$_billing_admin_search_lat       = filter_input(INPUT_POST, '_billing_admin_search_lat', FILTER_SANITIZE_SPECIAL_CHARS);
		$_billing_admin_search_lng       = filter_input(INPUT_POST, '_billing_admin_search_lng', FILTER_SANITIZE_SPECIAL_CHARS);
		$_billing_admin_address_map      = filter_input(INPUT_POST, '_billing_admin_address_map', FILTER_SANITIZE_SPECIAL_CHARS);
		$_billing_city                   = filter_input(INPUT_POST, '_billing_city', FILTER_SANITIZE_SPECIAL_CHARS);

		$billing_address['lat']          = !empty($_billing_admin_search_lat) ? $_billing_admin_search_lat : '';
		$billing_address['lng']          = !empty($_billing_admin_search_lng) ? $_billing_admin_search_lng : '';
		$billing_address['search_input'] = !empty($_billing_admin_address_map) ? $_billing_admin_address_map : '';
		$billing_address['search_city']  = !empty($_billing_city) ? $_billing_city : '';

		update_post_meta($post_id, '_billing_map_address', $billing_address);
	}

	/**
	 * Add address map data from submitted form
	 *
	 * @param  array $post_data
	 * @return array $post_data
	 */
	public function wc_checkout_address_map_data( $post_data ) {

		// get billing and shipping address map data
		$order_address_data = $this->get_order_address_map_data();
		// set checkout posted data billing and shipping address map
		$post_data['billing_address_map'] = $order_address_data['billing_data'];
		if (isset($post_data['ship_to_different_address']) && $post_data['ship_to_different_address'] ) {
			$post_data['shipping_address_map'] = $order_address_data['shipping_data'];
		} else {
			$shipping_data                     = ( is_user_logged_in() && get_user_meta(get_current_user_id(), 'shipping_address_map', true) ) ? get_user_meta(get_current_user_id(), 'shipping_address_map', true) : array();
			$post_data['shipping_address_map'] = $shipping_data;
		}

		return $post_data;
	}

	public function save_address_meta( $order_id ) {
		// Get billing and shipping address map data
		$order_address_data = $this->get_order_address_map_data();
		$hposupdate_meta    = wc_get_order($order_id);

		// Update and save billing address data
		$hposupdate_meta->set_billing_address_1($order_address_data['billing_data']['search_input']);
		$hposupdate_meta->update_meta_data('_billing_admin_address_map', $order_address_data['billing_data']['search_input']);

		if (!empty($order_address_data['shipping_data']['search_input'])) {
			// Update and save shipping address data
			$hposupdate_meta->set_shipping_address_1($order_address_data['shipping_data']['search_input']);
			$hposupdate_meta->update_meta_data('_shipping_admin_address_map', $order_address_data['shipping_data']['search_input']);
		} else {
			
			$hposupdate_meta->set_shipping_address_1($order_address_data['billing_data']['search_input']);
			$hposupdate_meta->update_meta_data('_shipping_admin_address_map', $order_address_data['billing_data']['search_input']);
		}
		$hposupdate_meta->save();
	}

	public function get_order_address_map_data() {
		// define variables
		$billing_data  = array();
		$shipping_data = array();
		// get billing data
		$billing_address_map_search_lat         = filter_input(INPUT_POST, 'billing_address_map_search_lat', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address_map_search_lng         = filter_input(INPUT_POST, 'billing_address_map_search_lng', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address_map_search_input       = filter_input(INPUT_POST, 'billing_address_map_search_input', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address_map_search_city        = filter_input(INPUT_POST, 'billing_address_map_search_city', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address_map_search_country     = filter_input(INPUT_POST, 'billing_address_map_search_country', FILTER_SANITIZE_SPECIAL_CHARS);
		
		$shipping_address_map_search_lat        = filter_input(INPUT_POST, 'shipping_address_map_search_lat', FILTER_SANITIZE_SPECIAL_CHARS);
		$shipping_address_map_search_lng        = filter_input(INPUT_POST, 'shipping_address_map_search_lng', FILTER_SANITIZE_SPECIAL_CHARS);
		$shipping_address_map_search_input      = filter_input(INPUT_POST, 'shipping_address_map_search_input', FILTER_SANITIZE_SPECIAL_CHARS);
		$shipping_address_map_search_city       = filter_input(INPUT_POST, 'shipping_address_map_search_city', FILTER_SANITIZE_SPECIAL_CHARS);
		$shipping_address_map_search_country    = filter_input(INPUT_POST, 'shipping_address_map_search_country', FILTER_SANITIZE_SPECIAL_CHARS);
		
		$billing_data['lat']                = !empty($billing_address_map_search_lat) ? $billing_address_map_search_lat : '';
		$billing_data['lng']                = !empty($billing_address_map_search_lng) ? $billing_address_map_search_lng : '';
		$billing_data['search_input']       = !empty($billing_address_map_search_input) ? $billing_address_map_search_input : '';
		$billing_data['_billing_address_1'] = !empty($billing_address_map_search_input) ? $billing_address_map_search_input : '';
		$billing_data['search_city']        = !empty($billing_address_map_search_city) ? $billing_address_map_search_city : '';
		$billing_data['search_country']     = !empty($billing_address_map_search_country) ? $billing_address_map_search_country : '';
		// get shipping data
		$shipping_data['lat']            = !empty($shipping_address_map_search_lat) ? $shipping_address_map_search_lat : '';
		$shipping_data['lng']            = !empty($shipping_address_map_search_lng) ? $shipping_address_map_search_lng : '';
		$shipping_data['search_input']   = !empty($shipping_address_map_search_input) ? $shipping_address_map_search_input : '';
		$shipping_data['search_city']    = !empty($shipping_address_map_search_city) ? $shipping_address_map_search_city : '';
		$shipping_data['search_country'] = !empty($shipping_address_map_search_country) ? $shipping_address_map_search_country : '';
		return array(
		'billing_data'  => $billing_data,
		'shipping_data' => $shipping_data,
		);
	}

	/**
	 * Validate custom billing and shipping address
	 */
	public function wc_validate_billing_shipping_address() {

		// validate shipping address and billing address is exist
		$ship_to_different_address          = filter_input(INPUT_POST, 'ship_to_different_address', FILTER_SANITIZE_SPECIAL_CHARS);
		$shipping_address_map_search_input  = filter_input(INPUT_POST, 'shipping_address_map_search_input', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address_map_search_input   = filter_input(INPUT_POST, 'billing_address_map_search_input', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address_map_search_lat     = filter_input(INPUT_POST, 'billing_address_map_search_lat', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address_map_search_lng     = filter_input(INPUT_POST, 'billing_address_map_search_lng', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address_map_find_location  = filter_input(INPUT_POST, 'billing_address_map_find_location', FILTER_SANITIZE_SPECIAL_CHARS);
		
		if (! empty($ship_to_different_address) ) {
			if (! empty($shipping_address_map_search_input) ) {
				if (empty(get_option('aafw_allow_manual_address')) ) {
					wc_add_notice(esc_html_e('<strong>Shipping Address</strong>,is a required field.', 'address-autocomplete-for-woocommerce'), 'error');
				}
			}
		} elseif (! empty($billing_address_map_search_input) || ! empty($billing_address_map_search_lat) || ! empty($billing_address_map_search_lng) ) {
			if (empty(get_option('aafw_allow_manual_address')) ) {
				wc_add_notice(esc_html_e('<strong>Billing Address</strong>,is a required field.', 'address-autocomplete-for-woocommerce'), 'error');
			}
		}
		if (! empty($billing_address_map_find_location) && 'false' === $billing_address_map_find_location && empty(get_option('aafw_allow_manual_address')) ) {
			wc_add_notice(esc_html_e('<strong>Location Not found </strong>,please try another location.', 'address-autocomplete-for-woocommerce'), 'error');
		}
	}

	public function change_billing_fields( $fields ) {
			
		$fields['billing']['billing_address_map'] = array(
		'type'     => 'address_map',
		/**
		 * Filter
		 * 
		 * @since 1.0.0
		 */
		'placeholder' => apply_filters('woo_billing_field_label', __('Enter Location', 'address-autocomplete-for-woocommerce')),
		'class'    => array( 'form-row-wide' ),
		/**
		* Filter
		* 
		* @since 1.0.0
		*/
		'label'    => apply_filters('woo_address_autocomplete_field_label', __('Selected Location', 'address-autocomplete-for-woocommerce')),
		'priority' => 20,
		);
		$fields['shipping']['shipping_address_map'] = array(
		'type'     => 'shipping_address_map',
		'class'    => array( 'form-row-wide' ),
		/**
		* Filter
		* 
		* @since 1.0.0
		*/
		'label'    => apply_filters('woo_shpping_address_autocomplete_field_label', __('Selected Location', 'address-autocomplete-for-woocommerce')),
		'priority' => 20,
		);
		$aafw_allow_manual_address = get_option('aafw_allow_manual_address');
		if ('1'  === $aafw_allow_manual_address ) {
			unset($fields['billing']['billing_address_1']);
			unset($fields['shipping']['shipping_address_1']);
			$fields['billing']['billing_state']['required']   = false;
			$fields['shipping']['shipping_state']['required'] = false;
			$fields['billing']['billing_state']['class']      = array( 'address-full-w' );
			$fields['shipping']['shipping_state']['class']    = array( 'address-full-w' );
		}
			
		
		return $fields;
	}
	public function get_allowed_countries( $countries ) {
		global $woocommerce;
		$selling_validate = get_option('aafw_enable_restriction');
		$countries_obj    = new WC_Countries();
		$all_countries    = $countries_obj->__get('countries');
		if ('1' === $selling_validate ) {
			return $countries;
		}
		return $all_countries;
	}
	/**
	 * Additional Feild Street
	 */
	public function street_checkout_fields( $fields ) {

		$aafw_enable_map = get_option('aafw_enable_map');
		if ('1' === $aafw_enable_map ) {
			$fields['billing']['billing_street_new'] = array(
			'class'       => array( 'form-row-wide' ),
			/**
			* Filter
			* 
			* @since 1.0.0
			*/
			'placeholder' => apply_filters('woo_street_field_label', __('Apartment,suite,unit,etc. (optional)', 'address-autocomplete-for-woocommerce')),
			'priority'    => 55,
			);
			 
		}
		return $fields;
	}

	/**
	 * Process the checkout
	 */
	public function street_checkout_field_update_order_meta( $order_id ) {

		$billing_street_new = filter_input(INPUT_POST, 'billing_street_new', FILTER_SANITIZE_SPECIAL_CHARS);
		$billing_address_1 = filter_input(INPUT_POST, 'billing_address_1', FILTER_SANITIZE_SPECIAL_CHARS);
		$hposupdate_meta = wc_get_order($order_id);

		if (! empty($billing_street_new) ) {
			$hposupdate_meta->update_meta_data('billing_street_new', $billing_street_new);
			
		}
		if (! empty($billing_address_1) ) {
			$hposupdate_meta->update_meta_data('billing_address_1', $billing_address_1);
			
		}
		$hposupdate_meta->save();
	}
	public function woo_address_appartment_no( $address_formats ) {
		$data = isset($_SERVER['QUERY_STRING']) ? sanitize_text_field(wp_unslash($_SERVER['QUERY_STRING'])) : '';
		parse_str($data, $query_params);
		if (is_admin() && isset($query_params['post']) && 'shop_order' === get_post_type(absint($query_params['post'])) && ! empty($address_formats) ) {
			$address_formats_updated = array();
			foreach ( $address_formats as $key => $value ) {
				
				$address_formats_updated[ $key ] = str_replace('{address_2}', "{apartment}\n{address_2}", $value);
				
			}
		
			$address_formats = $address_formats_updated;
		}
		return $address_formats;
	}

	public function woo_address_format_funct( $address_format_replacement ) {
		$data = isset($_SERVER['QUERY_STRING']) ? sanitize_text_field(wp_unslash($_SERVER['QUERY_STRING'])) : '';
		parse_str($data, $query_params);
		if (is_admin() && isset($query_params['post']) && 'shop_order' === get_post_type(absint($query_params['post'])) && ! empty($address_format_replacement) ) {

			$address_format_replacement['{apartment}'] = get_post_meta(absint($query_params['post']), 'billing_street_new', true);
		}
		return $address_format_replacement;
	}
}