<?php
use Automattic\WooCommerce\Blocks\Integrations\IntegrationInterface;

/**
 * Class for integrating with WooCommerce Blocks
 */
class Shipping_Workshop_Blocks_Integration implements IntegrationInterface {

	/**
	 * The name of the integration.
	 *
	 * @return string
	 */
	public function get_name() {
		return 'shipping-workshop';
	}

	/**
	 * When called invokes any initialization/setup for the integration.
	 */
	public function initialize() {
		$this->register_shipping_workshop_block_frontend_scripts();
		$this->register_shipping_workshop_block_editor_scripts();
		$this->register_shipping_workshop_block_editor_styles();
		$this->register_main_integration();
		/**
		 * [backend-step-09-extra-credit]
		 * 💰 Extra credit: Display the selection on the order confirmation page.
		 *
		 * To accomplish this, you'll need to add an action to the 
		 * `woocommerce_order_details_after_customer_details` hook. Uncomment the function call in 
		 * the next line, and implement the function body of `show_shipping_instructions_in_order_confirmation`.
		 * 
		 * 💡 Similar to [backend-step-08] respectively [backend-step-08-extra-credit], you'll need 
		 * to get the `shipping_workshop_alternate_shipping_instruction` and 
		 * `shipping_workshop_alternate_shipping_instruction_other_text` and print them out.
		 */

		/**
		 * [backend-step-10-extra-credit]
		 * 💰 Extra credit: Display the selection in the order confirmation email.
		 *
		 * To accomplish this, you'll need to add an action to the 
		 * `woocommerce_email_customer_details` hook. Uncomment the function call in 
		 * the next line, and implement the function body of `show_shipping_instructions_in_order_email`.
		 * 
		 * 💡 Similar to [backend-step-08] respectively [backend-step-08-extra-credit], you'll need 
		 * to get the `shipping_workshop_alternate_shipping_instruction` and 
		 * `shipping_workshop_alternate_shipping_instruction_other_text` and print them out.
		 */
	}

	/**
	 * Adds the address on the order confirmation page.
	 */
	private function show_shipping_instructions_in_order_confirmation() {
		/**
		 * [backend-step-09-extra-credit]
		 * Add your code here …
		 */
	}

	/**
	 * Adds the address on the order confirmation email.
	 */
	public function show_shipping_instructions_in_order_email() {
		/**
		 * [backend-step-10-extra-credit]
		 * Add your code here …
		 */
	}

	/**
	 * Registers the main JS file required to add filters and Slot/Fills.
	 */
	private function register_main_integration() {
		$script_path = '/build/index.js';
		$style_path  = '/build/style-index.css';

		$script_url = plugins_url( $script_path, __FILE__ );
		$style_url  = plugins_url( $style_path, __FILE__ );

		$script_asset_path = __DIR__ . '/build/index.asset.php';
		$script_asset      = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array(),
				'version'      => $this->get_file_version( $script_path ),
			);

		wp_enqueue_style(
			'shipping-workshop-blocks-integration',
			$style_url,
			array(),
			$this->get_file_version( $style_path )
		);

		wp_register_script(
			'shipping-workshop-blocks-integration',
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);
		wp_set_script_translations(
			'shipping-workshop-blocks-integration',
			'shipping-workshop',
			__DIR__ . '/languages'
		);
	}

	/**
	 * Returns an array of script handles to enqueue in the frontend context.
	 *
	 * @return string[]
	 */
	public function get_script_handles() {
		return array( 'shipping-workshop-blocks-integration', 'shipping-workshop-block-frontend' );
	}

	/**
	 * Returns an array of script handles to enqueue in the editor context.
	 *
	 * @return string[]
	 */
	public function get_editor_script_handles() {
		return array( 'shipping-workshop-blocks-integration', 'shipping-workshop-block-editor' );
	}

	/**
	 * An array of key, value pairs of data made available to the block on the client side.
	 *
	 * @return array
	 */
	public function get_script_data() {
		$data = array(
			'shipping-workshop-active' => true,
		);

		return $data;
	}

	public function register_shipping_workshop_block_editor_styles() {
		$style_path = '/build/style-index.css';

		$style_url = plugins_url( $style_path, __FILE__ );
		wp_enqueue_style(
			'shipping-workshop-block',
			$style_url,
			array(),
			$this->get_file_version( $style_path )
		);
	}

	public function register_shipping_workshop_block_editor_scripts() {
		$script_path       = '/build/shipping-workshop-block.js';
		$script_url        = plugins_url( $script_path, __FILE__ );
		$script_asset_path = __DIR__ . '/build/shipping-workshop-block.asset.php';
		$script_asset      = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array(),
				'version'      => $this->get_file_version( $script_asset_path ),
			);

		wp_register_script(
			'shipping-workshop-block-editor',
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);

		wp_set_script_translations(
			'shipping-workshop-block-editor',
			'shipping-workshop',
			__DIR__ . '/languages'
		);
	}

	public function register_shipping_workshop_block_frontend_scripts() {
		$script_path       = '/build/shipping-workshop-block-frontend.js';
		$script_url        = plugins_url( $script_path, __FILE__ );
		$script_asset_path = __DIR__ . '/build/shipping-workshop-block-frontend.asset.php';
		$script_asset      = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array(),
				'version'      => $this->get_file_version( $script_asset_path ),
			);

		wp_register_script(
			'shipping-workshop-block-frontend',
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);
		wp_set_script_translations(
			'shipping-workshop-block-frontend',
			'shipping-workshop',
			__DIR__ . '/languages'
		);
	}

	/**
	 * Get the file modified time as a cache buster if we're in dev mode.
	 *
	 * @param string $file Local path to the file.
	 * @return string The cache buster value to use for the given file.
	 */
	protected function get_file_version( $file ) {
		if ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG && file_exists( $file ) ) {
			return filemtime( $file );
		}
		return SHIPPING_WORKSHOP_VERSION;
	}
}
